﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Office.Interop.Excel;
using System.IO;

namespace ExcelOPC64
{
    public class ExcelTest : IDisposable
    {
        private Application exApp;
        private Workbook exBook;
        private Worksheet exSheet;

        private string _path;
        private Random _random;

        public ExcelTest(string path)
        {
            _path = path;
            _random = new Random();

            Launch();
        }

        private void Launch()
        {
            if (File.Exists(_path))
            {
                exApp = new Application();

                if (exApp != null)
                {
                    // Create Excel Book Object
                    exBook = exApp.Workbooks.Add(_path);
                    exSheet = (Worksheet)exBook.Worksheets[1];

                    exApp.Visible = true;
                    exApp.ScreenUpdating = true;
                    exApp.DisplayAlerts = false;
                    exApp.Calculation = XlCalculation.xlCalculationManual;
                }
            }
        }

        private void Close()
        {
            if (exBook != null)
            {
                exBook.Saved = true;
                //exBook.Save();
                exBook.SaveAs(_path, Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing, XlSaveAsAccessMode.xlNoChange, Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing);
                exBook.Close();

                System.Runtime.InteropServices.Marshal.FinalReleaseComObject(exBook);
                exBook = null;
            }

            if (exSheet != null)
            {
                System.Runtime.InteropServices.Marshal.FinalReleaseComObject(exSheet);
                exSheet = null;
            }

            if (exApp != null)
            {
                exApp.Quit();

                System.Runtime.InteropServices.Marshal.FinalReleaseComObject(exApp);
                exApp = null;
            }
        }

        public void Dispose()
        {
            Close();
        }

        public (double, double) WriteTest()
        {
            var randomNumberA = _random.NextDouble() * 10;
            var randomNumberB = _random.NextDouble() * 10;

            exSheet.Cells[10, "A"] = "Test values";
            exSheet.Cells[10, "B"] = randomNumberA;
            exSheet.Cells[10, "C"] = randomNumberB;

            return (randomNumberA, randomNumberB);
        }

        public (double, double) ReadTest()
        {
            var numberA = CheckDoubleValueFromExcel(Convert.ToDouble(exSheet.Cells[10, "B"].Value.ToString()), -1.0);
            var numberB = CheckDoubleValueFromExcel(Convert.ToDouble(exSheet.Cells[10, "C"].Value.ToString()), -1.0);

            return (numberA, numberB);
        }

        public (double, double) CalcaulationsTest()
        {
            exSheet.Cells[11, "A"] = "Calculations";
            exSheet.Cells[11, "B"].Formula = "=B10+C10";
            exSheet.Cells[11, "C"].Formula = "=B10-C10";

            exSheet.Calculate();

            var numberA = CheckDoubleValueFromExcel(Convert.ToDouble(exSheet.Cells[11, "B"].Value.ToString()), -1.0);
            var numberB = CheckDoubleValueFromExcel(Convert.ToDouble(exSheet.Cells[11, "C"].Value.ToString()), -1.0);

            return (numberA, numberB);
        }

        private double CheckDoubleValueFromExcel(double value, double fallbackValue)
        {
            switch (value)
            {
                case -2146826281:
                    throw new Exception("Cell contained #Div/0! error. Fallback value is used");
                case -2146826246:
                    throw new Exception("Cell contained #N/A error. Fallback value is used");
                case -2146826259:
                    throw new Exception("Cell contained #Name? error. Fallback value is used");
                case -2146826288:
                    throw new Exception("Cell contained #Null! error. Fallback value is used");
                case -2146826252:
                    throw new Exception("Cell contained #Num! error. Fallback value is used");
                case -2146826265:
                    throw new Exception("Cell contained #Ref! error. Fallback value is used");
                case -2146826273:
                    throw new Exception("Cell contained #Value! error. Fallback value is used");

                default:
                    return value;
            }
        }
    }
}
